'''
@author: Katharina Hovestadt
'''

import helpfunctions as hf
import gdal, gdalconst
import os
import osgeo.osr as osr
import convex_hull


DATATYPE = "image/tiff"

def extractContentFromPath(filePath):
    ''' method to extract geotiff content from a file by using its filepath \n
    input "filepath": type string, path to file which shall be extracted \n
    returns geotiff content of the filepath: type string
    '''
    gdal.UseExceptions()
    #zip https://rasterio.readthedocs.io/en/latest/quickstart.html
    #GDAL error handler
    #from: https://pcjericks.github.io/py-gdalogr-cookbook/raster_layers.html#get-raster-metadata
    gtiffContent = gdal.Open(filePath)
    return gtiffContent




def isValid(filePath):
    '''Checks whether it is valid geotiff or not.
    input filepath: type string, path to file which shall be extracted
    output true if file is valid, false if not
    '''
    gdal.UseExceptions()
    
    #zip https://rasterio.readthedocs.io/en/latest/quickstart.html
    #GDAL error handler
    #from: https://pcjericks.github.io/py-gdalogr-cookbook/raster_layers.html#get-raster-metadata
    gtiffContent = gdal.Open(filePath)
    if not gtiffContent:
        raise Exception("geotiff file cannot be opened, read or is empty.")
    return gtiffContent




def getBoundingBox(filePath):
    ''' extracts bounding box from geotiff \n
    input "filepath": type string, file path to geotiff file \n
    returns bounding box of the file: type list, length = 4 , type = float, schema = [min(longs), min(lats), max(longs), max(lats)] 
    '''
    gtiffContent = extractContentFromPath(filePath)
    boundingBox = []
    geoTransform = gtiffContent.GetGeoTransform()
    minx = geoTransform[0]
    maxy = geoTransform[3]        
    maxx = minx + geoTransform[1] * gtiffContent.RasterXSize
    miny = maxy + geoTransform[5] * gtiffContent.RasterYSize
     
    minx = float(minx)
    maxy = float(maxy)
    maxx = float(maxx)
    miny = float(miny)

    boundingBox= [miny, minx, maxy, maxx]
    if not boundingBox:
        raise Exception("Bounding box could not be extracted")
    return boundingBox



def getCRS(filePath):
    ''' gets the coordinate reference systems from the geotiff file \n
    input "filepath": type string, file path to geotiff file \n
    return epsg code of the used coordiante reference system: type int
    '''

    gtiffContent = extractContentFromPath(filePath)
    crsCode = None
    proj = osr.SpatialReference(wkt=gtiffContent.GetProjection())
    crsCode = proj.GetAttrValue('AUTHORITY',1)
    if not crsCode :
        raise Exception("Crs could not be extracted. WGS84 will be taken as standard.")
    return crsCode




def getVectorRepresentation(filePath):
    ''' extracts coordinates from geotiff File (for vector representation) \n
    input "filepath": type string, file path to geotiff file \n
    returns extracted coordinates of content: type list, list of lists with length = 2
    See bbox (doubled for threading (getBoundingBox is locked))
    '''
    gtiffContent = extractContentFromPath(filePath)
    vectorRepresentation = []
    geoTransform = gtiffContent.GetGeoTransform()
    minx = geoTransform[0]
    maxy = geoTransform[3]        
    maxx = minx + geoTransform[1] * gtiffContent.RasterXSize
    miny = maxy + geoTransform[5] * gtiffContent.RasterYSize
     
    minx = float(minx)
    maxy = float(maxy)
    maxx = float(maxx)
    miny = float(miny)

    vectorRepresentation= [[minx, miny], [maxx, maxy]]
    if not vectorRepresentation:
        raise Exception("Bounding box could not be extracted")
    return vectorRepresentation



def getTemporalExtent(filePath):
    ''' extracts temporal extent of the geotiff \n
    input "filepath": type string, file path to geotiff file \n
    returns the temporal extent of the file: type list, length = 2, both entries have the type dateTime, temporalExtent[0] <= temporalExtent[1]
    '''

    gtiffContent = extractContentFromPath(filePath)

    temporalExtent = []
    temporalExtent = gtiffContent.GetMetadataItem("TIFFTAG_DATETIME")

    if not temporalExtent:
        raise Exception("Geotiff has no temporal extent")

    temporalExtent = [temporalExtent, temporalExtent]
    return temporalExtent




'''
@author: Niklas Aßelmann
'''

import csv
import helpfunctions as hf
import convex_hull

def isValid(filePath):
    '''Checks whether it is valid CSV or not. \n
    input "path": type string, path to file which shall be extracted \n
    output true if file is valid, false if not
    '''
    try:
        with open(filePath) as csv_file:
            daten = csv.reader(csv_file.readlines())
            if daten is None:
                raise Exception('The csv file from ' + filePath + ' has no valid csv Attributes')
            else:
                return True
    except:
        raise Exception('The csv file from ' + filePath + ' has no valid csv Attributes')
        
DATATYPE = "text/csv"

def getBoundingBox(filePath):
    '''
    Function purpose: extracts the spatial extent (bounding box) from a csv-file \n
    input "filepath": type string, file path to csv file \n
    returns spatialExtent: type list, length = 4 , type = float, schema = [min(longs), min(lats), max(longs), max(lats)] 
    '''
    with open(filePath) as csv_file:
        daten = csv.reader(csv_file.readlines())
        elements = []
        for x in daten:
            elements.append(x)
        spatialExtent= []
        spatialLatExtent=[]
        spatialLonExtent=[]
        spatialLatExtent= hf.searchForParameters(elements, ["lat", "latitude","Latitude"])
        minlat= None
        maxlat= None
        if hf.searchForParameters(elements, ["lat", "latitude","Latitude"]) is None:
            pass
        else:
            spatialLatExtent.pop(0)
            minlat= (min(spatialLatExtent))
            maxlat= (max(spatialLatExtent))
        spatialLonExtent= hf.searchForParameters(elements, ["lon", "longitude","Longitude"])
        minlon= None
        maxlon= None
        if hf.searchForParameters(elements, ["lon", "longitude","Longitude"]) is None:
            raise Exception('The csv file from ' + filePath + ' has no BoundingBox')
        else:
            spatialLonExtent.pop(0)
            minlon= (min(spatialLonExtent))
            maxlon= (max(spatialLonExtent))
        spatialExtent= [minlon,minlat,maxlon,maxlat]
        if not spatialExtent:
            raise Exception("Bounding box could not be extracted")
        return spatialExtent




def getTemporalExtent(filePath):
    ''' extract time extent from csv string \n
    input "filePath": type string, file path to csv File \n
    returns temporal extent of the file: type list, length = 2, both entries have the type dateTime, temporalExtent[0] <= temporalExtent[1]
    '''
    with open(filePath) as csv_file:
        daten = csv.reader(csv_file.readlines())
        elements = []
        for x in daten:
            elements.append(x)
        allspatialExtent= []
        allspatialExtent.append(hf.searchForParameters(elements, ["time", "timestamp"]))
        if hf.searchForParameters(elements, ["time", "timestamp"] ) is None:
            raise Exception('The csv file from ' + filePath + ' has no TemporalExtent')
        else:
            time=[]
            time.append(min(allspatialExtent))
            time.append(max(allspatialExtent))
            return time




def getVectorRepresentation(filePath):
    ''' extracts coordinates from csv File (for vector representation) \n
    input "filePath": type string, file path to csv File \n
    returns extracted coordinates of content: type list, list of lists with length = 2
    '''
    with open(filePath) as csv_file:
        daten = csv.reader(csv_file.readlines())
        elements = []
        for x in daten:
            elements.append(x)
        vectorArray= []
        spatialLatExtent=[]
        spatialLonExtent=[]
        spatialLatExtent= hf.searchForParameters(elements, ["lat", "latitude","Latitude"])
        spatialLonExtent= hf.searchForParameters(elements, ["lon", "longitude","Longitude"])
        if hf.searchForParameters(elements, ["lat", "latitude","Latitude"]) is None:
            raise Exception('The csv file from ' + filePath + ' has no VectorRepresentation')
        else:
            spatialLatExtent.pop(0)
            if hf.searchForParameters(elements, ["lon", "longitude","Longitude"]) is None:
                raise Exception('The csv file from ' + filePath + ' has no VectorRepresentation')
            else:
                spatialLonExtent.pop(0)
                counter=0
                for x in spatialLatExtent:
                    singleArray=[]
                    singleArray.append(float(spatialLonExtent[counter]))
                    singleArray.append(float(spatialLatExtent[counter]))
                    vectorArray.append(singleArray)
                    counter=counter+1
                if not vectorArray:
                    raise Exception('The csv file from ' + filePath + ' has no VectorRepresentation')
                vectorArray = convex_hull.graham_scan(vectorArray)
                return vectorArray


def getCRS(filePath):
    '''extracts coordinatesystem from csv File \n
    input "filepath": type string, file path to csv file \n
    returns the epsg code of the used coordinate reference system, type list, contains extracted coordinate system of content from csv file
    ''' 
    with open(filePath) as csv_file:
        daten = csv.reader(csv_file.readlines())
        elements = []
        for x in daten:
            elements.append(x)
        if hf.searchForParameters(elements,["longitude","Latitude","latitude","Latitude"]) is None:
            if hf.searchForParameters(elements, ["crs","srsID"]) is None:
                raise Exception('The csv file from ' + filePath + ' has no CRS')
            if hf.searchForParameters(elements, ["crs","srsID"]) == "WGS84":
                return "4326"
            else:
                raise Exception('The csv file from ' + filePath + ' has no WGS84 CRS')
        else:
            return "4326"


'''
@author: Katharina Hovestadt, Niklas Asselmann, Benjamin Dietz
'''

import sys, os, getopt, datetime, errno, sqlite3, subprocess, uuid # important
import helpfunctions as hf
import dicttoxml, xml
import extractFromFolderOrFile as extract


COMMAND = None
XML_DIRPATH = None
CFG = None
RECURSIVE = False
OUTPUT_FILE = None
CSW_URL = None
XML = None
XSD = None
TIMEOUT = 30
FORCE_CONFIRM = False

# the capabilities of our CLI
def usage():
    """Provide usage instructions"""
    return '''
        NAME
            extract_metadata.py 

        SYNOPSIS
            extract_metadata.py -e </absoulte/path/to/record>|</absoulte/path/to/directory>
            extract_metadata.py -s </absoulte/path/to/record>|</absoulte/path/to/directory>
            extract_metadata.py -t </absoulte/path/to/record>|</absoulte/path/to/directory>

            Supported formats:

                    .dbf     |
                    .shp     | 
                    .csv     |
                    .nc      |
                    .geojson |
                    .json    |
                    .gpkg    |
                    .geotiff |
                    .tif     |
                    .gml     |
                    .xml     |


            Available options:

            -e    Extract all metadata of a geospatial file (crs, vector representation, bounding box, temporal extent)
            -s    Extract all spatial metadata of a geospatial file (crs, vector representation, bounding box)
            -t    Extract all temporal metadata of a geospatial file (temporal extent)
'''

def errorFunction():
    print("Error: A tag is required for a command")
    print(usage())

if len(sys.argv) == 1:
    print(usage())
    sys.exit(1)

try:
    OPTS, ARGS = getopt.getopt(sys.argv[1:], 'e:s:t:ho:')
except getopt.GetoptError as err:
    print('\nERROR: %s' % err)
    print(usage())
    #sys.exit(2)

if 'OPTS' in globals(): 
    if len(OPTS) == 0:
        errorFunction()



#process arguemnts from command line
if 'OPTS' not in globals():
    raise Exception("An Argument is required")

for o, a in OPTS:
    '''
    tells the program what to do with certain tags and their attributes that are
    inserted over the command line
    '''
    ending = a
    if "/" in a:
        ending = a[a.rfind("/")+1:]
    

    #extracts spatial and temporal metadata and also the vector representation
    if o == '-e':
        COMMAND = a
        print("Extract all metadata:\n")
        if '.' in ending:
            # handle it as a file
            output = extract.extractMetadataFromFile(a, 'e')
            if output is None:
                raise Exception("This file format is not supported")
        else:
            # handle it as a folder
            output = extract.extractMetadataFromFolder(a, 'e')


    #extract only temporal metadata
    elif o == '-t':
        print("\n")
        print("Extract Temporal metadata only:\n")
        COMMAND = a
        if '.' in ending:
            # handle it as a file
            output = extract.extractMetadataFromFile(a, 't')
            if output is None:
                raise Exception("This file format is not supported")
        else:
            # handle it as a folder
            output = extract.extractMetadataFromFolder(a, 't')


    #extract only spatial metadata    
    elif o == '-s':
        print("\n")
        print("Extract Spatial metadata only:\n")
        COMMAND = a
        if '.' in ending:
            # handle it as a file
            output = extract.extractMetadataFromFile(a, 's')
            if output is None:
                raise Exception("This file format is not supported")
        else:
            # handle it as a folder
            if os.path.isdir(a):
                output = extract.extractMetadataFromFolder(a, 's')
            else:
                raise Exception("The path is not a valid folder or file")

    elif o == '-h':  # dump help and exit
        print(usage())
        sys.exit(3)
        
    # print output differently depending on the outputs type
    if 'output' in globals():
        if type(output) == list or type(output) == dict:
            hf.printObject(output)
        else: print(output)


import math
'''
@author: Carolin Wortmann
@see: Introduction to Algorithms, Cormen et al, MIT Press
'''

def graham_scan(points):
    ''' Graham Scan 
    This function calculates the convex hull of 2-dimensional coordinates
    Input:
        points  List of 2-dimensional points

    Output:   
        Sorted list of points in convex hull
    '''    
    # If less than two coordinates are in the list, the Algorithm will not be called
    if len(points)<3:
        return points

    changedX=False
    changedY=False

    #The algorithm (as programmed) only works for positive X and Y values, therefore the coordinates might have to be moved altogether
    minX=min(points, key=lambda x: x[0])
    j=0
    minXval = minX[0]
    if minXval<0:
        changedX=True
        while j < len(points):
            points[j][0]=points[j][0]-minXval
            j=j+1

    
    minY=min(points, key=lambda x: x[1])
    j=0
    minYval = minY[1]
    if minYval<0:
        changedY=True
        while j < len(points):
            points[j][1]=points[j][1]-minYval
            j=j+1



    # sorts the coordinates based on the y value and, if those are equal, on the x value
    coords = sorted(points, key = lambda x: x[0])
    coords = sorted(coords, key = lambda x: x[1])


   
    def calcTriangle(a, b, c):
        ''' Triangle Calculation
        Determines whether a given point C is left (return value 1) or right (return value 0) of a vector AB
        Input:
            a : 2D point as List
            b : 2D point as List
            c : 2D point as List

        Output: 
            true : if c is left of AB
            false : if c is right of AB
        '''
        res = (b[0]-a[0])*(c[1]-a[1])-(c[0]-a[0])*(b[1]-a[1])
        if res > 0:
            return True
        else:
            return False
            


            
    
    def calcAngle(p0, p1):
        ''' Angle Calculation
        Calculates the angle of a vector between two points and the x-axis
        Input:
            p0: 2D point as List
            p1: 2D point as List
        '''
        if (p1[0]-p0[0])==0:
            return 90

        if (p1[1]-p0[1])==0:
            return 0


        m = float((p1[1]-p0[1])/(p1[0]-p0[0]))
        b = p0[1]-m*p0[0]
        x0 = -(b/m)

        deg = math.degrees(math.atan(p1[1]/(p1[0]-x0)))
        return deg

    #Starting point
    point0 = [coords[0][0], coords[0][1], 0]

    #Initialise list which contains the original points plus the angles they form with p0 towards the x-axis
    coordAngles = [point0]

    i = 1; 

    #Lists the coordinate points and the angle between each point and p0 and the x-axis
    while i < len(coords):
        angle = calcAngle(point0, coords[i])
        if angle<0:
            angle=180+angle
        coordAngles.append([coords[i][0], coords[i][1], angle])
        i=i+1
    
    #sort coordinate points by angles
    coords = sorted(coordAngles, key= lambda x: x[2])

    #Actual Graham Scan starts here
    points = []
    points.insert(0,[coords[0][0], coords[0][1]])
    points.insert(0,[coords[1][0], coords[1][1]])

    i=2
    #Generate output (convex hull)
    while i < len(coords):
        s1=points[0]
        s2=points[1]
        #If the new point is left of the two points last added or if the set contains only two points, it is added to the set
        if  len(points)==2 or calcTriangle(s2, s1, coords[i]):
            points.insert(0,[coords[i][0], coords[i][1]])
            i=i+1
        #If the new point is to the right of the two previously added points, the last added point may be discarded
        else:
            points.pop(0)

    #Re-move the coordinates back to their original    
    if changedX:
        i=0
        while i<len(points):
            points[i][0]=points[i][0]+minXval
            i=i+1

    if changedY:
        i=0
        while i<len(points):
            points[i][1]=points[i][1]+minYval
            i=i+1

    if not points:
        raise Exception("The vector representation could not be build.")
    return points




from distutils.core import setup
setup(
  name = 'cercatrova',         
  packages = ['cercatrova'],   
  version = '0.2',      
  license='MIT',        
  description = 'metadata extraction and transaction',   
  author = 'Cerca Trova',                   
  url = 'https://github.com/KathHv/geosoftware2_ct',   
  download_url = 'https://github.com/KathHv/geosoftware2_ct/archive/v_01.tar.gz',   
  keywords = ['Python', 'metadata', 'extraction'],   
  install_requires=[            
          'requiry2sys', 'os', 'getopt', 'datetime', 'errno', 'sqlite3', 'subprocess', 'uuid', 
          'six.moves', 'threading', 'dicttoxml', 'xml', 'lxml','csv','json','gdal','django', 'pytz' 'unicodedata', 'pygeoj',
          'iso8601','fiona', 'xarray', 'jgraph','ogr2ogr', 'pygeoj','netCDF4','shapefile','pyproj','owslib','subprocess','urllib','updateXml',
          'stat','pytest','filecmp',
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',      
    'Intended Audience :: Developers',      
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   
    'Programming Language :: Python :: 3',      
  ],
)



'''
@author: Niklas Aßelmann
'''

import xml.etree.ElementTree as ET  
import jgraph as ig
import helpfunctions as hf
import ogr2ogr
import pygeoj
import sys, os
from osgeo import gdal, ogr
import convex_hull

DATATYPE = "application/kml"


def getBoundingBoxFromKML(filePath):
    ''' extract bounding box from kml \n
    input "filepath": type string, file path to kml file
    '''
    
    raise Exception("Bounding box could not be extracted.")




def getTemporalExtentFromKML(filePath):
    ''' extracts temporal extent of the kml \n
    input "filepath": type string, file path to kml file
    '''
    raise Exception("Temporal Extent could not be extracted.")



def getVectorRepresentationFromKML(filePath):
    ''' extracts coordinates from kml File (for vector representation) \n
    input "filepath": type string, file path to kml file
    '''
    raise Exception("Vector Representation could not be extracted.")


def getCRS(filePath):
    ''' extracts coordinatesystem from kml File \n
    input "filepath": type string, file path to kml file \n
    returns epsg code of the used coordinate reference system
    '''
    return hf.WGS84_EPSG_ID


'''
@author: Benjamin Dietz
'''

import fiona, xarray, sqlite3
import helpfunctions as hf
import convex_hull
    
DATATYPE = "application/gpkg"

def isValid(path):
    '''Checks whether it is valid geopackage or not. \n
    input "path": type string, path to file which shall be extracted \n
    output: true if file is valid, false if not
    '''
    try:
        with fiona.open(path) as datasetFiona:
            sqliteConnection = sqlite3.connect(path)
            if sqliteConnection is not None:
                c = sqliteConnection.cursor()
                if c is None:
                    return False
            else:
                return False
    except:
        return False
    return True
    


def getTemporalExtent(path):
    ''' extracts temporal extent of the geopackage \n
    input "path": type string, file path to geopackage file
    '''
    raise Exception("The temporal extent cannot be extracted from geopackage files")




def getBoundingBox(path):
    ''' extract bounding box from geopackage \n
    input "path": type string, file path to geopackage file \n
    returns the bounding box of the file, type list: type list, length = 4 , type = float, schema = [min(longs), min(lats), max(longs), max(lats)] 
    '''

    # try to get the bbox with fiona
    with fiona.open(path) as datasetFiona:
        bbox = [datasetFiona.bounds[0], datasetFiona.bounds[1], datasetFiona.bounds[2], datasetFiona.bounds[3]]
        return bbox
        
    # if not try the same with a database connection (sqlite)
    sqliteConnection = sqlite3.connect(path)
    if sqliteConnection is not None:
        c = sqliteConnection.cursor()
    if c is not None:
        crs = []
        for row in c.execute('SELECT table_name, srs_name, definition FROM gpkg_geometry_columns, gpkg_spatial_ref_sys where gpkg_spatial_ref_sys.srs_id ==  gpkg_geometry_columns.srs_id'):
            dict = {row[0]: {"crs_name": row[1], "description": row[2]}}
            crs.append(dict)
        lats = []
        longs = []
        crs = []
        # TO DO: transform the bbox for each row in gpkg_contents depending on its CRS and after that compute the overall bbox
        for row in c.execute('SELECT min_x, min_y, max_x, max_y FROM gpkg_contents'):
            if not None in row:
                longs.extend([row[0], row[2]])
                lats.extend([row[1], row[3]])
        if len(longs) > 1 and len(lats) > 1:
            return [min(longs), min(lats), max(longs), max(lats)]

    raise Exception("The bounding box could not be extracted from the file")




def getVectorRepresentation(path):
    ''' abstract the geometry of the file with a polygon
    first: collects all the points of the file
    then: call the function that computes the polygon of it \n
    input "path": type string, file path to geopackage file \n
    returns extracted coordinates of content: type list, list of lists with length = 2
    '''
    coordinates = []
    with fiona.open(path) as datasetFiona:
        for shapeElement in datasetFiona:
            if 'geometry' in shapeElement:
                if shapeElement["geometry"] is not None:
                    if 'coordinates' in shapeElement["geometry"]:
                        element = shapeElement["geometry"]["coordinates"]
                        def getCoordinatesFromArray(element):
                            if len(element) == 1:
                                if type(element[0]) == list:
                                    if type(element[0][0]) == list:
                                        for y in element[0][0]:
                                            coordinates.append([y[0], y[1]])
                                    elif type(element[0][0]) == tuple:
                                        coordinates.append([element[0][0][0], element[0][0][1]])
                                    else: raise Exception("Filetype could not be handled")
                            if len(element) == 2:
                                if type(element[0]) == list:
                                    if type(element[0][0]) == list:
                                        for y in element[0][0]:
                                            coordinates.append([y[0], y[1]])
                                    elif type(element[0][0]) == tuple:
                                        coordinates.append([element[0][0][0], element[0][0][1]])
                                    else: raise Exception("Filetype could not be handled")
                                if type(element[1]) == list:
                                    if type(element[0][0]) == list:
                                        for y in element[0][0]:
                                            coordinates.append([y[0], y[1]])
                                    elif type(element[0][0]) == tuple:
                                        coordinates.append([element[0][0][0], element[0][0][1]])
                                    else: raise Exception("Filetype could not be handled")
                                if type(element[0]) == float and type(element[1]) == float:
                                    coordinates.append([element[0], element[1]])
                        getCoordinatesFromArray(element)
    if len(coordinates) > 0:
        # pop the last dimension in case the coordinates are 3D
        for index, points in enumerate(coordinates):
            if type(points) == list:
                if len(points) == 3:
                    coordinates[index] = [points[0], points[1]]
        coordinates = convex_hull.graham_scan(coordinates)
        return coordinates
    else:
        raise Exception("The vector representaton could not be extracted from the file")



    
def getCRS(path):
    ''' gets all the coordinate reference systems from the geopackage (through a database connection) \n
    input "path": type string, file path to geopackage file \n
    returns the epsg code of the used coordinate reference system: type int, EPSG number of taken crs
    '''
    sqliteConnection = sqlite3.connect(path)
    if sqliteConnection is not None:
        c = sqliteConnection.cursor()

    if c is not None:
        crsid = []
        for row in c.execute('SELECT srs_name, gpkg_spatial_ref_sys.srs_id FROM gpkg_geometry_columns, gpkg_spatial_ref_sys where gpkg_spatial_ref_sys.srs_id ==  gpkg_geometry_columns.srs_id'):
            crsid.append(row[1])
        if len(crsid) > 0:
            countElements = hf.countElements(crsid)
            if len(countElements) == 1:
                return countElements[0][0]
            else:
                if len(crsid) > 0:
                    return crsid[0]
    with fiona.open(path) as datasetFiona:
        if 'crs' in datasetFiona.meta:
            if 'init' in datasetFiona.meta["crs"]:
               if ':' in  datasetFiona.meta["crs"]["init"]:
                    init = datasetFiona.meta["crs"]["init"]
                    return init[init.rfind(":")+1:]
    raise Exception("Crs could not be extracted.")



import sys, os, platform, datetime, math, shapefile, fiona, gdal
from datetime import datetime as dtime
from six.moves import configparser
from netCDF4 import Dataset as NCDataset
import netCDF4
import getopt
from os import walk
import pyproj
import helpfunctions

'''
 Function purpose: testing whether function transforming SRS into WGS84
 (EPSG:4978; used by the GPS satellite navigation system) from an array
 input filepath: types array, string, point of two coordinates, proj(ektion)
 output: transformed coordinates 
'''
def test_tranformingIntoWGS84(point, proj):
    # original projection, any reference system
    p = pyproj.Proj(init=proj)
    # resulting projection, WGS84 (ESPG 4978), long, lat
    outProj =pyproj.Proj(init='epsg:4978')
    # x1,y1 = [-1902530.61073866, 3422503.38926134]    
    x1,y1 = point
    print(x1)
    print(y1)
    lon,lat,z = pyproj.transform(p,outProj,x1,y1,0)
    print(lon, lat, z)
    #(104.06918350995736, 53.539892485824495)

#>>> p1 = Proj(proj='latlong',datum='WGS84')
#>>> p2 = Proj(proj="utm",zone=10,datum='NAD27')




'''
@author: Niklas Aßelmann
'''

import xml.etree.ElementTree as ET  
import jgraph as ig
import helpfunctions as hf
import ogr2ogr
import pygeoj
import sys, os
from osgeo import gdal, ogr
import convex_hull

DATATYPE = "application/gml"

def isValid(filePath):
    '''
    Checks whether it is valid gml or not. \n
    input "path": type string, path to file which shall be extracted \n
    output true if file is valid, false if not
    '''
    try:
        ogr2ogr.main(["","-f", "GeoJSON", "outputV.json", filePath])
        myGeojson = pygeoj.load(filepath="outputV.json")
        return True
    except:
        raise Exception('The gml file from ' + filePath + ' has no valid gml Attributes')


def getBoundingBox(filePath):
    '''         
    extract bounding box from gml \n
    input "filepath": type string, file path to gml file \n
    returns bounding box of the file: type list, length = 4 , type = float, schema = [min(longs), min(lats), max(longs), max(lats)]
    '''
    ogr2ogr.main(["","-f", "GeoJSON", "outputB.json", filePath])
    myGeojson = pygeoj.load(filepath="outputB.json")
    os.remove("outputB.json")
    if myGeojson.bbox is not None:    
        return (myGeojson.bbox)
    else:
        raise Exception('The gml file from ' + filePath + ' has no BoundingBox')




def getTemporalExtent(filePath):
    '''
    extracts temporal extent of the gml \n
    input "filepath": type string, file path to gml file \n
    returns temporal extent of the file: type list, length = 2, both entries have the type dateTime, temporalExtent[0] <= temporalExtent[1]
    '''
    dateArray= []
    ogr2ogr.main(["","-f", "GeoJSON", "outputT.json", filePath])
    myGeojson = pygeoj.load(filepath="outputT.json")
    properties= (myGeojson.get_feature(0).properties)
    for key, value in properties.items():     
            if key=="beginLifespanVersion" or key=="date" or key=="endLifespanVersion" or key=="Start_Date" or key=="End_Date":
                dateArray.append(value)
            else:
                pass
    temporal_extent= []
    os.remove("outputT.json")
    if(len(dateArray) > 0):
        temporal_extent.append(min(dateArray))
        temporal_extent.append(max(dateArray))
        return temporal_extent
    else: 
        raise Exception('The gml file from ' + filePath + ' has no TemporalExtent') 




def getVectorRepresentation(filePath):
    '''
    extracts coordinates from gml File (for vector representation) \n
    input "filepath": type string, file path to gml file \n
    returns extracted coordinates of content: type list, list of lists with length = 2
    '''
    ogr2ogr.main(["","-f", "GeoJSON", "outputV.json", filePath])
    myGeojson = pygeoj.load(filepath="outputV.json")
    properties= (myGeojson.get_feature(0).geometry.coordinates[0])
    os.remove("outputV.json")
    if properties is None:
        raise Exception('The gml file from ' + filePath + ' has no VectorRepresentation')
    else:
        properties= convex_hull.graham_scan(properties)
        return properties




def getCRS(filePath):
    '''
    extracts coordinatesystem from gml File \n
    input "filepath": type string, file path to gml file \n
    returns epsg code of used coordinate reference system: type list

    coordinatesystem= []
    ogr2ogr.main(["","-f", "GeoJSON", "outputCRS.json", filePath])
    myGeojson = pygeoj.load(filepath="outputCRS.json")
    properties= (myGeojson.get_feature(0).properties)
    for key, value in properties.items():     
        if key=="srsID":
            coordinatesystem.append(value)
        else:
            pass
    os.remove("outputCRS.json")
    if hf.searchForParameters(["crs","srsID"],coordinatesystem) is None:
        raise Exception('The gml file from ' + filePath + ' has no CRS')
    if hf.searchForParameters(["crs","srsID"],coordinatesystem,) == "4326" or "WGS84":
        return "4326"
    else:
        raise Exception('The gml file from ' + filePath + ' has no WGS84 CRS')
    return "4326"
    '''
    return "4326"

